/*
 * Copyright (C) 1995 M. Hauber, Ch. Schneider, G. Caronni
 * See COPYING for more details
 */
/* (c) by G. Caronni in 1995                              18.11.95
 *
 * Contains an interface to arbitrary MPI packages
 *
 */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <assert.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "cert_defs.h"
#include "cert_int.h"
#include "gmp/gmp.h"


int int_init(void **where)
{
    *where=(void *)malloc(sizeof(MP_INT));
    mpz_init((MP_INT *)*where);
    return 0;
}

void int_drop(void *mpi)
{
    mpz_clear((MP_INT *)mpi);
    free(mpi);
}

int int_modexp(void *res, void *g, void *j, void *p)
{
    mpz_powm(res,g,j,p);
    return 0;
}

int int_extract_raw(unsigned char **where, void *mpi, int fix_size)
{
    MP_INT extr;
    int len,i;

    mpz_init_set(&extr,(MP_INT *)mpi);
    len=(mpz_sizeinbase(&extr,2)+7)/8;
    *where=(unsigned char *)malloc(len<fix_size?fix_size:len);
    memset(*where,0,len<fix_size?fix_size:len);
    for(i=(len<fix_size?fix_size:len)-1;i>=0;i--) {
        (*where)[i]= (unsigned char) (mpz_get_ui(&extr) & 0xff);
        mpz_div_2exp(&extr,&extr,(unsigned long int)8);
    }
    return len;
}

char *int_extract_hex(void *mpi)
{
    int len,i;
    unsigned char *raw, *s;
    char *buf, *t;

    len=int_extract_raw(&raw,mpi,0);
    buf=(char *)malloc(2*len+1);
    for(t=buf,s=raw,i=0;i<len;i++,t+=2,s++) sprintf(t,"%.2X",*s);
    free(raw);
    return buf;
}

int int_copy(void *to, void *from)
{
    mpz_set((MP_INT *)to,(MP_INT *)from);
    return 0;
}

int int_compare(void *a, void *b)
{
    return mpz_cmp((MP_INT *)a,(MP_INT *)b);
}

int int_set_str(void *val, char *str, int base) 
{
    return mpz_set_str(val,str,base);
}

int int_bitsize(void *mpi)
{
    return mpz_sizeinbase((MP_INT *)mpi,2);
}
