/*
 * Copyright (C) 1995 M. Hauber, Ch. Schneider, G. Caronni
 * See COPYING for more details
 */
#include <sys/types.h>
#include <string.h>
#include <stdio.h>

#include "skip_defs.h"
#include "id.h"

int id_printhex(FILE *f, u_char *val, int len)
{
  int i;

  for (i = 0; i < len; i++)
    fprintf(f, "%02x", *val++);

  return 0;
}

int id_printnicehex(FILE *f, u_char *val, int len)
{
  fprintf(f, "0x");
  return id_printhex(f, val, len);
}

int id_scanhex(char *val, char *buf, int l)
{
  int used;

  for (used = 0; used < l; used++, val += 2)
  {
    static char *hexdigits = "0123456789abcdefABCDEF", *hd1, *hd2;

    if ((hd1 = strchr(hexdigits, val[0])) && (hd2 = strchr(hexdigits, val[1])))
    {
      int d1 = hd1 - hexdigits;
      int d2 = hd2 - hexdigits;
      if (d1 > 15)  /* Uppercase */
	d1 -= 6;
      if (d2 > 15)  /* Uppercase */
	d2 -= 6;
      *buf++ = (d1 << 4) | d2;
    }
    else
    {
      fprintf(stderr, "skipd: malformed hex number at '%s'\n", val);
      used = -1;
      break;
    }
  }

  return used;
}

int id_saveids(FILE *f, skip_id_t *id)
{
  fprintf(f, "%u-", id->type[0]);
  id_printhex(f, id->id[0], sizeof(id->id[0]));
  fprintf(f, ",%u-", id->type[1]);
  id_printhex(f, id->id[1], sizeof(id->id[1]));
  fprintf(f, ":\n");

  return 0;
}

int id_parseids(char *name, skip_id_t *id)
{
  int t0, t1, l0, l1;
  char *id1, *id2, *tmp;

  if ((id1 = strtok(name, "-")) == NULL)
    return -1;
  t0 = atoi(id1);

  if ((id1 = strtok(NULL, ", ")) == NULL)
    return -1;

  tmp = id1 + strlen(id1) + 1;

  while ((*tmp == ' ') || (*tmp == ',')) tmp++;

  if ((id2 = strtok(tmp, "-")) == NULL)
    return -1;
  t1 = atoi(id2);

  if ((id2 = strtok(NULL, ":")) == NULL)
    return -1;

  l0 = strlen(id1);
  l1 = strlen(id2);

  id->type[0] = t0, id->type[1] = t1;

  if ((l0 & 1) || (l1 & 1))
  {
    fprintf(stderr, "skipd: malformed hex number, length is odd\n");
    return -1;
  }

  l0 /= 2, l1 /= 2;

  if ((l0 > SKIP_ID_MAXLEN) || (l1 > SKIP_ID_MAXLEN))
  {
    fprintf(stderr, "skipd: master key id too large\n");
    return -1;
  }

  return ((id_scanhex(id1, id->id[0], l0) < 0) ||
	  (id_scanhex(id2, id->id[1], l1) < 0)) ? -1 : 0;
}
